﻿unit CMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants, Androidapi.JNI.Net, Androidapi.Helpers, Androidapi.JNI.App,
  Androidapi.JNI.Provider, Androidapi.IOUtils, System.Messaging,
  Androidapi.JNI.GraphicsContentViewText, Androidapi.JNI.Os, FMX.Layouts,
  FMX.ListBox, FMX.Types, FMX.Controls, FMX.Controls.Presentation, FMX.StdCtrls,
  Androidapi.JNI.androidx,
  Androidapi.JNI.JavaTypes, Androidapi.JNIBridge, System.Permissions,
  FMX.Forms, FMX.Graphics, FMX.Dialogs;

type
  TFormMain = class(TForm)
    btnRequest: TButton;
    btnNavigate: TButton;
    lbFiles: TListBox;
    procedure btnRequestClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnNavigateClick(Sender: TObject);
  private
    { Private declarations }
    FMessageSubscriptionID: Integer;
    procedure NavigateDocument(document: JDocumentFile);
  public
    { Public declarations }
    files: Androidapi.JNIBridge.TJavaObjectArray<JDocumentFile>;
    function OnActivityResult(RequestCode, ResultCode: Integer;
      Data: JIntent): Boolean;
    procedure HandleActivityMessage(const Sender: TObject; const M: TMessage);
  end;

var
  FormMain: TFormMain;

const
  RequestCode = 88;

implementation

uses
  IOUtils;

{$R *.fmx}

function GetDataTreeUri: JNet_uri;
begin
  Result := TJDocumentsContract.JavaClass.buildTreeDocumentUri
    (StringToJString('com.android.externalstorage.documents'),
    StringToJString('primary:Android/data'));
end;

// 判断是否已经获取了Data权限
function isDataGrant(): Boolean;
var
  Permissions: JList;
  I: Integer;
  permission: JUriPermission;
  treeUri: JNet_uri;
  count: Integer;
begin
  Permissions := TAndroidHelper.Context.getContentResolver.
    getPersistedUriPermissions;
  treeUri := GetDataTreeUri;
  count := Permissions.size;
  Log.d(Format('Data Access Permisson count:%d', [count]));
  for I := 0 to count - 1 do
  begin
    permission := TJUriPermission.Wrap(Permissions.get(I));
    if permission.isReadPermission and permission.getUri.equals(treeUri) then
    begin
      Log.d('Data Access Granted');
      Exit(True);
    end;
  end;
  Result := false;
end;

// 获取指定目录的权限
procedure requestDataPermission(code: Integer);
var
  // uri: string;
  uri1: JNet_uri;
  // parse: Jnet_Uri;
  intent: JIntent;
begin
  uri1 := TJDocumentsContract.JavaClass.buildDocumentUri
    (StringToJString('com.android.externalstorage.documents'),
    StringToJString('primary:Android/data'));

  intent := TJIntent.JavaClass.init
    (TJIntent.JavaClass.ACTION_OPEN_DOCUMENT_TREE);
  // intent.addFlags(TJIntent.JavaClass.FLAG_GRANT_READ_URI_PERMISSION or
  // TJIntent.JavaClass.FLAG_GRANT_WRITE_URI_PERMISSION or
  // TJIntent.JavaClass.FLAG_GRANT_PERSISTABLE_URI_PERMISSION or
  // TJIntent.JavaClass.FLAG_GRANT_PREFIX_URI_PERMISSION);
  intent.putExtra(TJDocumentsContract.JavaClass.EXTRA_INITIAL_URI,
    TJParcelable.Wrap(uri1));

  SharedActivity.startActivityForResult(intent, code);
end;

procedure TFormMain.btnRequestClick(Sender: TObject);
begin
  // 请求权限
  if not isDataGrant then
    requestDataPermission(RequestCode);
end;

procedure TFormMain.btnNavigateClick(Sender: TObject);
var
  path: string;
  treeUri: JNet_uri;
  document: JDocumentFile;
  fs:TStringDynArray;
  I:Integer;
begin
  //
  if isDataGrant then
  begin
    if lbFiles.Selected = nil then
    begin

      // get data uri
      treeUri := GetDataTreeUri;
      document := TJDocumentFile.JavaClass.fromTreeUri
        (SharedActivityContext, treeUri);
      NavigateDocument(document);
    end
    else
    begin
      document := files[lbFiles.ItemIndex];
      NavigateDocument(document);
    end;
  end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FMessageSubscriptionID := TMessageManager.DefaultManager.SubscribeToMessage
    (TMessageResultNotification, HandleActivityMessage);

  PermissionsService.RequestPermissions
    ([JStringToString(TJManifest_permission.JavaClass.READ_EXTERNAL_STORAGE),
    JStringToString(TJManifest_permission.JavaClass.WRITE_EXTERNAL_STORAGE)],
    procedure(const APermissions: TArray<string>;
      const AGrantResults: TArray<TPermissionStatus>)
    begin

    end);
end;

procedure TFormMain.HandleActivityMessage(const Sender: TObject;
const M: TMessage);
begin
  if M is TMessageResultNotification then
    OnActivityResult(TMessageResultNotification(M).RequestCode,
      TMessageResultNotification(M).ResultCode,
      TMessageResultNotification(M).Value);
end;

procedure TFormMain.NavigateDocument(document: JDocumentFile);
var
  I: Integer;
  f: string;
  input:JInputStream;
  output:JOutputStream;
  buffer:TJavaArray<Byte>;
  read:Integer;
  fs:TFileStream;
begin
  if document.isDirectory then
  begin
    lbFiles.clear;
    files := document.listFiles;
    for I := 0 to files.Length - 1 do
    begin
      if files[I].isDirectory then
        f:='>>';
      f := f+JStringToString(files[I].getName);
      lbFiles.Items.Add(f);
    end;
  end
  else
  begin
    MessageDlg('Do you want to copy file to /sdcard/Movies/ folder, yes or no?',
      System.UITypes.TMsgDlgType.mtInformation,
      [System.UITypes.TMsgDlgBtn.mbYes, System.UITypes.TMsgDlgBtn.mbNo], 0,
      procedure(const AResult: TModalResult)
      begin
        case AResult of
          { Detect which button was pushed and show a different message }
          mrYes:
            begin
              // pressed yes\
              //copy the file to /sdcard/Movies folder
              input:=TAndroidHelper.Context.getContentResolver.openInputStream(document.getUri);
              buffer:=TJavaArray<Byte>.Create(2048);
              fs:=TFileStream.Create('/sdcard/Movies/test.mp4', fmCreate);
              while true do
              begin
                read:=input.read(buffer);
                if read=-1 then
                  break;
                fs.Write(buffer.Data^, read);
              end;

              fs.free;
            end;
          mrNo:
            begin
              // pressed no
            end;
        end;
      end);
  end;
end;

function TFormMain.OnActivityResult(RequestCode, ResultCode: Integer;
Data: JIntent): Boolean;
var
  uri: JNet_uri;
begin
  Result := false;

  TMessageManager.DefaultManager.Unsubscribe(TMessageResultNotification,
    FMessageSubscriptionID);
  FMessageSubscriptionID := 0;

  if RequestCode = RequestCode then
  begin
    if ResultCode = TJActivity.JavaClass.RESULT_OK then
    begin
      if Assigned(Data) then
      begin
        uri := Data.getData();
        if uri <> nil then
        begin
          // 保存权限
          TAndroidHelper.Context.getContentResolver.takePersistableUriPermission
            (uri, // Data.getFlags() and
          (TJIntent.JavaClass.FLAG_GRANT_READ_URI_PERMISSION
            // or TJIntent.JavaClass.FLAG_GRANT_WRITE_URI_PERMISSION
            ));
          Log.d('Data Access Granted saved');

        end;
      end;
    end
    else if ResultCode = TJActivity.JavaClass.RESULT_CANCELED then
    begin
      //
    end;
    Result := True;
  end;
end;

end.
